/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
/**
 * Class representing state changes for the death of a player.
 *
 * @author Matthew Johnson <autoumpire@matthew.ath.cx>
 */
public class Death extends StateChange
{
   public Death(Assassin killer, Assassin victim)
   {
      super();
      this.ass = victim;
      this.pluginName = "vanilla";
      this.field = "killed-by";
      this.value = killer;
   }
   public Death(AssassinViewer killer, AssassinViewer victim)
   {
      super();
      this.ass = (Assassin) victim.getEntity();
      this.pluginName = "vanilla";
      this.field = "killed-by";
      this.value = (Assassin) killer.getEntity();
   }
   Death(int scid)
   {
      super(scid);
   }
   Death(Integer scid)
   {
      super(scid);
   }
   
   /**
    * Retrieve a viewer for the assassin who is dying.
    * @return The viewer representing the victim.
    */
   public AssassinViewer getVictim() throws AssassinSQLException
   {
      if (null == ass)
         readDetails();
      return new AssassinViewer(ass);
   }

   /**
    * Retrieve a viewer for the assassin responsible for this death.
    * @return The viewer representing the killer.
    */
   public AssassinViewer getKiller() throws AssassinSQLException
   {
      if (null == value)
         readDetails();
      return new AssassinViewer((Assassin) value);
   }
}

