/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.List;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * Viewer for the Player class
 * @see Player
 */
public class PlayerViewer extends CanonicalViewer
{
   PlayerViewer(Player p)
   {
      this.ent = p;
   }
   
   /**
    * Retrieve the real name of this player.
    * @return The player's name.
    */
   public String getName() throws AssassinSQLException
   {
      return ((Player)this.ent).getName();
   }

   /**
    * Retrieve the &quot;random other information&quot; field for this player.
    * @return The player's notes.
    */
   public String getNotes() throws AssassinSQLException
   {
      return ((Player)this.ent).getNotes();
   }
   public String getEmail() throws AssassinSQLException
   {
      return ((Player)this.ent).getEmail();
   }
   public String getCollege() throws AssassinSQLException
   {
	   return ((Player)this.ent).getCollege();
   }
   public String getAddress() throws AssassinSQLException
   {
	   return ((Player)this.ent).getAddress();
   }
   public String getWaterStatus() throws AssassinSQLException
   {
	   return ((Player)this.ent).getWaterStatus();
   }
   public String getPassword() throws AssassinSQLException
   {
	   return ((Player)this.ent).getPassword();
   }
   /**
    * Return a list of all assassins associated with this player. Each assassin
    * represents an incarnation of the player into a game-role (which may span
    * several actual deaths and resurrections of the 'same' assassin).
    * @return A list of {@link AssassinViewer} objects for the assassins
    *    associated with this player.
    */
   public List getAssassins() throws AssassinSQLException
   {
      return ((Player)this.ent).getAssassins();
   }
}
